<?php

/**
 * @file
 * Hooks and preprocess functions for the Blazy module.
 */

use Drupal\Component\Serialization\Json;
use Drupal\blazy\Blazy;

/**
 * Prepares variables for elevatezoomplus.html.twig templates.
 *
 * @see Drupal\commerce_product\ProductVariationFieldRenderer
 */
function template_preprocess_elevatezoomplus(array &$variables) {
  // Supported themes: Blazy (field|item_list), slick_wrapper, gridstack.
  $element = &$variables['element'];
  $variables['settings'] = [];
  foreach (['blazy', 'build', 'context', 'settings'] as $key) {
    if (isset($element['#' . $key])) {
      $settings = isset($element['#' . $key]['settings']) ? $element['#' . $key]['settings'] : [];
      $variables['settings'] = ($key == 'build' || $key == 'context') ? $settings : $element['#' . $key];
      break;
    }
  }

  // Reset theme attributes as we are a wrapper/ container here.
  $settings = $variables['settings'];
  $json = elevatezoomplus()->getOptions($settings);

  // @todo move it back to Blazy.
  // Was already excluded at Blazymanager #505 and passed into
  // #container_attributes below, gone at 2021, likely changed somewhere.
  $ajax_class = isset($element['#ajax_replace_class']) ? $element['#ajax_replace_class'] : '';
  if (empty($ajax_class) && isset($element['#attributes'], $element['#attributes']['class'])) {
    foreach ($element['#attributes']['class'] as $key => $class) {
      if (mb_strpos($class, 'product--variation-field--variation') !== FALSE) {
        $ajax_class = $class;
        break;
      }
    }
  }

  $variables['attributes'] = isset($element['#container_attributes']) ? (array) $element['#container_attributes'] : [];
  $variables['attributes']['data-elevatezoomplus'] = Json::encode($json);
  $variables['content_attributes'] = ['data-blazy' => ''];

  // Checks if Splide/ Slick asNavFor, or regular Slick, Blazy Grid, etc.
  // Only provides static stage if not using Slick asNavFor.
  $variables['stage'] = empty($settings['nav']) ? elevatezoomplus_build($settings) : [];
  $children = isset($element['#children']) ? $element['#children'] : [];
  $content = '';

  if ($children) {
    $content = $children->__toString();

    // @todo remove when Blazy got #container_attributes back post 2.5+.
    if ($ajax_class) {
      $content = str_replace($ajax_class, '', $content);
      $variables['attributes']['class'][] = $ajax_class;
    }
  }

  $variables['content']['#markup'] = $content;
}

/**
 * Build the main gallery image/ stage if not using Slick asNavFor.
 *
 * @todo remove first_item, first_uri (BC with older Blazy 2) at Blazy 3.x.
 */
function elevatezoomplus_build(array $settings = []) {
  $uri   = isset($settings['_uri']) ? $settings['_uri'] : (isset($settings['first_uri']) ? $settings['first_uri'] : '');
  $item  = isset($settings['first_item']) ? $settings['first_item'] : '';
  $item  = isset($settings['_item']) ? $settings['_item'] : $item;
  $url   = empty($settings['box_style']) ? file_url_transform_relative(file_create_url($uri)) : blazy()->entityLoad($settings['box_style'])->buildUrl($uri);
  $style = empty($settings['thumbnail_style']) ? '' : $settings['thumbnail_style'];
  $param = ['image_style' => $style, 'lazy' => 'blazy', 'uri' => $uri];
  $id    = Blazy::getHtmlId('elevatezoomplus');
  $zoom  = ['data-zoom-image' => $url];
  $attrs = ['class' => ['elevatezoomplus elevatezoomplus--stage'], 'id' => $id] + $zoom;
  $build = ['item' => $item, 'settings' => $param, 'item_attributes' => $attrs];

  return [
    '#type'     => 'inline_template',
    '#template' => '{{ stage }}',
    '#context'  => ['stage' => blazy()->getBlazy($build)],
    '#attached' => blazy()->attach($param + $settings),
  ];
}
