/**
 * @file
 * Provides ElevateZoomPlus loader.
 */

(function ($, Drupal, drupalSettings) {

  'use strict';

  var _wrapper = 'elevatezoomplus-wrapper';
  var _unload = false;

  /**
   * ElevateZoomPlus public methods.
   *
   * @namespace
   */
  Drupal.elevateZoomPlus = Drupal.elevateZoomPlus || {
    itemSelector: '.elevatezoomplus',
    defaults: drupalSettings.elevateZoomPlus || {},
    ez: null
  };

  /**
   * ElevateZoomPlus gallery item functions.
   *
   * @param {int} i
   *   The index of the current element.
   * @param {HTMLElement} elm
   *   The .elevatezoomplus-wrapper HTML element.
   */
  function doZoomWrapper(i, elm) {
    var me = Drupal.elevateZoomPlus;
    var options = me.defaults;
    var $elm = $(elm);
    var $target = $(me.itemSelector, elm);
    var $slick = $('.slick--main', elm);
    var $splide = $('.splide--main', elm);
    var $slider = $('> .slick__slider', $slick);
    var dataConfig = $elm.data('elevatezoomplus') || {};
    var hasMedia = $('.media--video', elm).length;
    var initialZoom = $splide.data('initialZoom') || $slick.data('initialZoom') || 0;
    var isSlick = $slick.length;

    // Bail out if we have no config available.
    if (!dataConfig) {
      return;
    }

    // Attach our behaviors.
    options = $.extend({}, options, dataConfig);

    if (options.behaviors) {
      check(options.behaviors);
    }

    if (hasMedia && Drupal.blazyBox) {
      Drupal.blazyBox.attach();
    }

    // Integrates with Splide/ Slick with asNavFor.
    if (!$slider.length) {
      $slider = $splide;
    }

    if ($slider.length) {
      // Cannot hook into Slick init, due to re-positioning.
      $target = $('.slide--' + initialZoom + ' ' + me.itemSelector, $slider);
      $target.ezPlus(options);

      me.ez = $target.data('ezPlus');

      if (isSlick) {
        $slider.on('beforeChange.ez', function () {
          close();
        });

        $slider.on('afterChange.ez', function () {
          close();
          $target = $('.slick-current ' + me.itemSelector, $slider);
          $target.ezPlus(options);

          me.ez = $target.data('ezPlus');
        });
      }
      else {
        var splide = $slider[0].splide;

        if (splide) {
          splide.on('moved.ez', function () {
            close();
          });

          splide.on('active.ez', function () {
            close();
            $target = $('.slide.is-active ' + me.itemSelector, $slider);
            $target.ezPlus(options);

            me.ez = $target.data('ezPlus');
          });
        }
      }
    }
    else {
      // Integrates with Slick without asNavFor, Blazy Grid, Gridstack, etc.
      $target.ezPlus(options);
      me.ez = $target.data('ezPlus');
    }

    /**
     * Closes the zoom window, can only exist one at a time.
     */
    function close() {
      if (me.ez) {
        me.ez.closeAll();
      }
      if ($('.ZoomContainer').length) {
        $('.ZoomContainer').remove();
      }
    }

    /**
     * Triggers video zoom on a click event.
     */
    function triggerZoom() {
      var $item = $(this);
      var media = $item.data('media') || false;
      var video = media && media.type === 'video';

      // Build own video fullscreen as ElevateZoomPlus is not for video.
      if (video && Drupal.blazyBox) {
        Drupal.blazyBox.open($item.parent().attr('href'));
      }
    }

    $('.media--switch--elevatezoomplus', elm).on('click.ez', triggerZoom);
    $elm.addClass('elevatezoomplus-wrapper--on');
  }

  /**
   * Workarounds for screwed-up context after AJAX contents.
   *
   * @param {array} behaviors
   *   The currently given Drupal.behaviors.
   * @todo remove post Splide and Slick updates.
   */
  function check(behaviors) {
    if (_unload && behaviors) {
      var len = behaviors.length;
      for (var i = 0; i < len; i++) {
        var item = behaviors[i];
        if (Drupal.behaviors[item]) {
          Drupal.behaviors[item].attach(document);
        }
      }
      _unload = false;
    }
  }

  /**
   * Attaches ElevateZoomPlus behavior to HTML element.
   *
   * @type {Drupal~behavior}
   */
  Drupal.behaviors.elevateZoomPlus = {
    attach: function (context) {

      // Context is unreliable with AJAX contents like product variations, etc.
      context = context instanceof HTMLDocument ? context : document;

      $('.' + _wrapper, context).once(_wrapper).each(doZoomWrapper);

    },
    detach: function (context, settings, trigger) {
      _unload = trigger === 'unload';
    }
  };

}(jQuery, Drupal, drupalSettings));
